<?php

namespace Drupal\Tests\k8s\Traits;

use Drupal\Component\Utility\Random;

/**
 * The trait creating mock data for k8s testing.
 */
trait K8sTestMockTrait {

  /**
   * Update getNodes in mock data.
   *
   * @param array $data
   *   The mock data.
   */
  protected function updateNodesMockData(array $data) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();

    $nodes = [];
    foreach ($data as $node_data) {
      $nodes[] = [
        'metadata' => [
          'name' => $node_data['name'],
          'creationTimestamp' => date('Y/m/d H:i:s'),
          'labels' => [
            'label1' => 'label-' . $random->string(8, TRUE),
          ],
          'annotations' => [
            'annotation1' => 'annotation-' . $random->string(8, TRUE),
          ],
        ],
        'spec' => [
          'podCIDR' => $random->name(8, TRUE),
          'providerID' => $random->name(8, TRUE),
          'unschedulable' => FALSE,

        ],
        'status' => [
          'conditions' => [
            [
              'type' => 'Ready',
            ],
          ],
          'addresses' => [
            [
              'type' => 'type-' . $random->name(8, TRUE),
              'address' => implode(
                '.',
                [rand(0, 254), rand(0, 255), rand(0, 255), rand(1, 255)]
              ),
            ],
          ],
          'nodeInfo' => [
            'machineID' => 'machine-' . $random->name(16, TRUE),
            'systemUUID' => 'system-' . $random->name(16, TRUE),
            'bootID' => 'boot-' . $random->name(16, TRUE),
            'kernelVersion' => 'kernel-' . $random->name(8, TRUE),
            'osImage' => 'os-' . $random->name(8, TRUE),
            'containerRuntimeVersion' => 'version-' . $random->name(8, TRUE),
            'kubeletVersion' => 'kubelet-' . $random->name(8, TRUE),
            'kubeProxyVersion' => 'kubeProxy-' . $random->name(8, TRUE),
            'operatingSystem' => 'linux',
            'architecture' => 'arc-' . $random->name(8, TRUE),
          ],
        ],
      ];
    }
    $mock_data['getNodes'] = $nodes;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createNamespace in mock data.
   *
   * @param array $namespace
   *   The namespace mock data.
   */
  protected function addNamespaceMockData(array $namespace) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_namespace = [
      'metadata' => [
        'name' => $namespace['name'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['createNamespace'] = $create_namespace;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateNamespace in mock data.
   *
   * @param array $namespace
   *   The namespace mock data.
   */
  protected function updateNamespaceMockData(array $namespace) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_namespace = [
      'metadata' => [
        'name' => $namespace['name'],
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['updateNamespace'] = $update_namespace;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteNamespace in mock data.
   *
   * @param array $namespace
   *   The namespace mock data.
   */
  protected function deleteNamespaceMockData(array $namespace) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_namespace = [
      'metadata' => [
        'name' => $namespace['name'],
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['deleteNamespace'] = $delete_namespace;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deletePod in mock data.
   *
   * @param array $pod
   *   The pod mock data.
   */
  protected function deletePodMockData(array $pod) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_pod = [
      'metadata' => [
        'name' => $pod['name'],
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['deletePod'] = $delete_pod;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createPod in mock data.
   *
   * @param array $pod
   *   The pod mock data.
   */
  protected function addPodMockData(array $pod) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_pod = [
      'metadata' => [
        'name' => $pod['name'],
        'namespace' => $pod['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['createPod'] = $create_pod;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $pod['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_pods = [];
    $get_pods[] = $create_pod;
    $get_pods[0]['spec'] = [
      'containers' => [],
      'nodeName' => 'node-name' . $random->name(8, TRUE),
    ];
    $get_pods[0]['status'] = [
      'containerStatuses' => [],
      'phase' => 'Active',
      'qosClass' => 'BestEffort',
      'podIP' => '10.0.0.1',
    ];
    $mock_data['getPods'] = $get_pods;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updatePod in mock data.
   *
   * @param array $pod
   *   The pod mock data.
   */
  protected function updatePodMockData(array $pod) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_pod = [
      'metadata' => [
        'name' => $pod['name'],
        'namespace' => $pod['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['updatePod'] = $update_pod;

    $get_pods = [];
    $get_pods[] = $update_pod;
    $get_pods[0]['spec'] = [
      'containers' => [],
      'nodeName' => 'node-name' . $random->name(8, TRUE),
    ];
    $get_pods[0]['status'] = [
      'containerStatuses' => [],
      'phase' => 'Active',
      'qosClass' => 'BestEffort',
      'podIP' => '10.0.0.1',
    ];
    $mock_data['getPods'] = $get_pods;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update getPodLogs in mock data.
   *
   * @param string $logs
   *   The pod mock data.
   */
  protected function getPodLogsMockData($logs) {
    $mock_data = $this->getMockDataFromConfig();

    $mock_data['getPodLogs'] = $logs;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createDeployment in mock data.
   *
   * @param array $deployment
   *   The pod mock data.
   */
  protected function addDeploymentMockData(array $deployment) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_deployment = [
      'metadata' => [
        'name' => $deployment['name'],
        'namespace' => $deployment['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'status' => [
        'phase' => 'Active',
      ],
    ];
    $mock_data['createDeployment'] = $create_deployment;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $deployment['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_deployments = [];
    $get_deployments[] = $create_deployment;
    $get_deployments[0]['spec'] = [
      'strategy' => ['type' => 'rollingUpdate'],
      'revisionHistoryLimit' => 10,
    ];
    $get_deployments[0]['status'] = [
      'availableReplicas' => 3,
      'collisionCount' => 1,
      'observedGeneration' => 5,
      'readyReplicas' => 2,
      'replicas' => 2,
      'unavailableReplicas' => 1,
      'updatedReplicas' => 2,
    ];
    $mock_data['getDeployments'] = $get_deployments;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateDeployment in mock data.
   *
   * @param array $deployment
   *   The deployment mock data.
   */
  protected function updateDeploymentMockData(array $deployment) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_deployment = [
      'metadata' => [
        'name' => $deployment['name'],
        'namespace' => $deployment['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['updateDeployment'] = $update_deployment;

    $get_deployments = [];
    $get_deployments[] = $update_deployment;
    $get_deployments[0]['spec'] = [
      'strategy' => ['type' => 'rollingUpdate'],
      'revisionHistoryLimit' => 10,
    ];
    $get_deployments[0]['status'] = [
      'availableReplicas' => 3,
      'collisionCount' => 1,
      'observedGeneration' => 5,
      'readyReplicas' => 2,
      'replicas' => 2,
      'unavailableReplicas' => 1,
      'updatedReplicas' => 2,
    ];
    $mock_data['getDeployments'] = $get_deployments;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteDeployment in mock data.
   *
   * @param array $deployment
   *   The deployment mock data.
   */
  protected function deleteDeploymentMockData(array $deployment) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_deployment = [
      'metadata' => [
        'name' => $deployment['name'],
      ],
    ];
    $mock_data['deleteDeployment'] = $delete_deployment;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createService in mock data.
   *
   * @param array $service
   *   The pod mock data.
   */
  protected function addServiceMockData(array $service) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_service = [
      'metadata' => [
        'name' => $service['name'],
        'namespace' => $service['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'ports' => [],
        'selector' => [],
        'clusterIP' => '10.10.0.1',
        'type' => 'ClusterIP',
        'sessionAffinity' => 'None',
      ],
    ];
    $mock_data['createService'] = $create_service;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $service['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_services = [];
    $get_services[] = $create_service;
    $mock_data['getServices'] = $get_services;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateService in mock data.
   *
   * @param array $service
   *   The service mock data.
   */
  protected function updateServiceMockData(array $service) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_service = [
      'metadata' => [
        'name' => $service['name'],
        'namespace' => $service['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'ports' => [],
        'selector' => [],
        'clusterIP' => '10.10.0.1',
        'type' => 'ClusterIP',
        'sessionAffinity' => 'None',
      ],
    ];
    $mock_data['updateService'] = $update_service;

    $get_services = [];
    $get_services[] = $update_service;
    $mock_data['getServices'] = $get_services;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteService in mock data.
   *
   * @param array $service
   *   The service mock data.
   */
  protected function deleteServiceMockData(array $service) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_service = [
      'metadata' => [
        'name' => $service['name'],
      ],
    ];
    $mock_data['deleteService'] = $delete_service;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createCronJob in mock data.
   *
   * @param array $cron_job
   *   The pod mock data.
   */
  protected function addCronJobMockData(array $cron_job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_cron_job = [
      'metadata' => [
        'name' => $cron_job['name'],
        'namespace' => $cron_job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'schedule' => '* * */1 * *',
        'suspend' => FALSE,
        'concurrencyPolicy' => 'Allow',
        'startingDeadlineSeconds' => 10,
      ],
      'status' => [
        'lastScheduleTime' => date('Y/m/d H:i:s'),
        'active' => [],
      ],
    ];
    $mock_data['createCronJob'] = $create_cron_job;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $cron_job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_cron_jobs = [];
    $get_cron_jobs[] = $create_cron_job;
    $mock_data['getCronJobs'] = $get_cron_jobs;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateCronJob in mock data.
   *
   * @param array $cron_job
   *   The cron job mock data.
   */
  protected function updateCronJobMockData(array $cron_job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_cron_job = [
      'metadata' => [
        'name' => $cron_job['name'],
        'namespace' => $cron_job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'schedule' => '* * */1 * *',
        'suspend' => FALSE,
        'concurrencyPolicy' => 'Allow',
        'startingDeadlineSeconds' => 10,
      ],
      'status' => [
        'lastScheduleTime' => date('Y/m/d H:i:s'),
        'active' => [],
      ],
    ];
    $mock_data['updateCronJob'] = $update_cron_job;

    $get_cron_jobs = [];
    $get_cron_jobs[] = $update_cron_job;
    $mock_data['getCronJobs'] = $get_cron_jobs;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteCronJob in mock data.
   *
   * @param array $cron_job
   *   The cron job mock data.
   */
  protected function deleteCronJobMockData(array $cron_job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_cron_job = [
      'metadata' => [
        'name' => $cron_job['name'],
      ],
    ];
    $mock_data['deleteCronJob'] = $delete_cron_job;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createJob in mock data.
   *
   * @param array $job
   *   The pod mock data.
   */
  protected function addJobMockData(array $job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_job = [
      'metadata' => [
        'name' => $job['name'],
        'namespace' => $job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'completions' => 1,
        'parallelism' => 1,
        'template' => [
          'spec' => [
            'containers' => [
              [
                'image' => 'perl',
              ],
            ],
          ],
        ],
      ],
      'status' => [
        'active' => 0,
        'succeeded' => 1,
        'failed' => 0,
      ],
    ];
    $mock_data['createJob'] = $create_job;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_jobs = [];
    $get_jobs[] = $create_job;
    $mock_data['getJobs'] = $get_jobs;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateJob in mock data.
   *
   * @param array $job
   *   The job mock data.
   */
  protected function updateJobMockData(array $job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_job = [
      'metadata' => [
        'name' => $job['name'],
        'namespace' => $job['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'completions' => 1,
        'parallelism' => 1,
        'template' => [
          'spec' => [
            'containers' => [
              [
                'image' => 'perl',
              ],
            ],
          ],
        ],
      ],
      'status' => [
        'active' => 0,
        'succeeded' => 1,
        'failed' => 0,
      ],
    ];
    $mock_data['updateJob'] = $update_job;

    $get_jobs = [];
    $get_jobs[] = $update_job;
    $mock_data['getJobs'] = $get_jobs;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteJob in mock data.
   *
   * @param array $job
   *   The job mock data.
   */
  protected function deleteJobMockData(array $job) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_job = [
      'metadata' => [
        'name' => $job['name'],
      ],
    ];
    $mock_data['deleteJob'] = $delete_job;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update createResourceQuota in mock data.
   *
   * @param array $resource_quota
   *   The pod mock data.
   */
  protected function addResourceQuotaMockData(array $resource_quota) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $create_resource_quota = [
      'metadata' => [
        'name' => $resource_quota['name'],
        'namespace' => $resource_quota['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'status' => [
        'hard' => [
          'cpu' => '10',
          'memory' => '20Gi',
          'pods' => '10',
        ],
        'used' => [
          'cpu' => '100m',
          'memory' => '1Gi',
          'pods' => '1',
        ],
      ],
    ];
    $mock_data['createResourceQuota'] = $create_resource_quota;

    $get_namespaces[] = [
      'metadata' => [
        'name' => $resource_quota['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
    ];
    $mock_data['getNamespaces'] = $get_namespaces;

    $get_resource_quotas = [];
    $get_resource_quotas[] = $create_resource_quota;
    $mock_data['getResourceQuotas'] = $get_resource_quotas;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update updateResourceQuota in mock data.
   *
   * @param array $resource_quota
   *   The resource quota mock data.
   */
  protected function updateResourceQuotaMockData(array $resource_quota) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $update_resource_quota = [
      'metadata' => [
        'name' => $resource_quota['name'],
        'namespace' => $resource_quota['post_data']['namespace'],
        'creationTimestamp' => date('Y/m/d H:i:s'),
      ],
      'spec' => [
        'completions' => 1,
        'parallelism' => 1,
        'template' => [
          'spec' => [
            'containers' => [
              [
                'image' => 'perl',
              ],
            ],
          ],
        ],
      ],
      'status' => [
        'active' => 0,
        'succeeded' => 1,
        'failed' => 0,
      ],
    ];
    $mock_data['updateResourceQuota'] = $update_resource_quota;

    $get_resource_quotas = [];
    $get_resource_quotas[] = $update_resource_quota;
    $mock_data['getResourceQuotas'] = $get_resource_quotas;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update deleteResourceQuota in mock data.
   *
   * @param array $resource_quota
   *   The resource quota mock data.
   */
  protected function deleteResourceQuotaMockData(array $resource_quota) {
    $mock_data = $this->getMockDataFromConfig();

    $random = new Random();
    $delete_resource_quota = [
      'metadata' => [
        'name' => $resource_quota['name'],
      ],
    ];
    $mock_data['deleteResourceQuota'] = $delete_resource_quota;
    $this->updateMockDataToConfig($mock_data);
  }

}
