<?php

namespace Drupal\cloud\Plugin\Menu;

use Drupal\cloud\Plugin\CloudConfigPluginManagerInterface;
use Drupal\Core\Menu\MenuLinkDefault;
use Drupal\Core\Menu\StaticMenuLinkOverridesInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Class adjusts the cloud service provider title and link.
 */
class CloudProviderLink extends MenuLinkDefault {

  /**
   * {@inheritdoc}
   */
  protected $overrideAllowed = [
    'menu_name' => 'main',
    'expanded' => TRUE,
  ];

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Creates a LocalTasksBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Menu\StaticMenuLinkOverridesInterface $static_override
   *   The static override storage.
   * @param \Drupal\cloud\Plugin\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The local task manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   A logger instance.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, StaticMenuLinkOverridesInterface $static_override, CloudConfigPluginManagerInterface $cloud_config_plugin_manager, RouteMatchInterface $route_match, LoggerChannelFactoryInterface $logger_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $static_override);
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->routeMatch = $route_match;
    $this->logger = $logger_factory->get('CloudProviderLink');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('menu_link.static.overrides'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('current_route_match'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    $title = $this->t('Cloud service providers');
    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    if ($cloud_context != NULL) {
      try {
        $this->cloudConfigPluginManager->setCloudContext($cloud_context);
        $config = $this->cloudConfigPluginManager->loadConfigEntity();
        $title = $config->getName();
      }
      catch (\Exception $e) {
        $this->logger->error($this->t('Cannot load cloud service provider.'));
      }
    }
    return $title;
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteName() {
    $route = 'view.cloud_config.list';
    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    if ($cloud_context != NULL) {
      try {
        $this->cloudConfigPluginManager->setCloudContext($cloud_context);
        $route = $this->cloudConfigPluginManager->getInstanceCollectionTemplateName();
      }
      catch (\Exception $e) {
        $this->logger->error($this->t('Cannot load cloud service provider.'));
      }
    }
    return $route;
  }

  /**
   * {@inheritdoc}
   */
  public function getRouteParameters() {
    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    if ($cloud_context != NULL) {
      return ['cloud_context' => $cloud_context];
    }
    return parent::getRouteParameters();
  }

}
